@@ -3,7 +3,7 @@ package Data::ULID;
 use strict;
 use warnings;
 
-our $VERSION = '1.1.3_03';
+our $VERSION = '1.2.1';
 
 use base qw(Exporter);
 our @EXPORT_OK = qw/ulid binary_ulid ulid_date ulid_to_uuid uuid_to_ulid/;
@@ -118,8 +118,8 @@ sub _fix_ts {
     my $ts = shift;
 
     if (CAN_SKIP_BIGINTS) {
-        $ts *= 1000;
-        return pack 'Nn', int($ts / (2 << 15)), $ts % (2 << 15);
+        $ts = int($ts * 1000);
+        return pack 'Nn', $ts >> 16, $ts & 0xffff;
     } else {
         $ts .= '000';
         $ts =~ s/\.(\d{3}).*$/$1/;
@@ -132,7 +132,7 @@ sub _unfix_ts {
 
     if (CAN_SKIP_BIGINTS) {
         my ($high, $low) = unpack 'Nn', $ts;
-        return ($high * (2 << 15) + $low) / 1000;
+        return (($high << 16) + $low) / 1000;
     } else {
         $ts = Math::BigInt->from_bytes($ts);
         $ts =~ s/(\d{3})$/.$1/;
@@ -151,16 +151,18 @@ sub _decode {
 }
 
 sub _zero_pad {
-    my ($value, $mul, $char) = @_;
-    $char ||= '0';
+    # this function is used a lot. Keep it as lean as possible
+    # my ($value, $character_multiplier, $padding_character) = @_;
+    my $value = shift;
 
-    my $padded = length($value) % $mul;
+    my $padded = length($value) % $_[0];
     return $value if $padded == 0;
 
+    $_[1] ||= 0;
     my $padding = substr $value, 0, $padded, '';
 
-    return $value if $padding eq $char x $padded;
-    return $char x ($mul - $padded) . $padding . $value;
+    return $value if $padding eq $_[1] x $padded;
+    return $_[1] x ($_[0] - $padded) . $padding . $value;
 }
 
 ### BASE32 ENCODER / DECODER
@@ -272,16 +274,20 @@ timestamp part.
 
  $ulid = ulid();
 
-Given a DateTime object as parameter, the function will set the timestamp part
-based on that:
-
- $ulid = ulid($datetime_obj);
-
 Given a binary ULID as parameter, it returns the same ULID in canonical
 format:
 
  $ulid = ulid($binary_ulid);
 
+Given a DateTime object as parameter, the function will set the timestamp part
+of the returned ULID accordingly:
+
+ $ulid = ulid($datetime_obj);
+
+This functionality obviously requires the DateTime module to be installed.
+Basic usage of C<Data::ULID> does not require this module, but it is
+recommended.
+
 =head2 Binary representation
 
 The binary representation of a ULID is 16 octets long, with each component in
@@ -298,7 +304,8 @@ ULID in the canonical representation and convert it to binary:
 =head2 Datetime extraction
 
 The C<ulid_date()> function takes a ULID (canonical or binary) and returns
-a DateTime object corresponding to the timestamp it encodes.
+a DateTime object corresponding to the timestamp it encodes. As above, this
+requires the DateTime module to be installed.
 
  $datetime = ulid_date($ulid);
 
